#!/usr/bin/python3
import subprocess
import time

import gi
from gi.repository import GLib, UDisks


UDI = "/org/freedesktop/UDisks2/block_devices/sdb"
ESP_GUID = 'C12A7328-F81F-11D2-BA4B-00A0C93EC93B'

udisksclient = UDisks.Client.new_sync()
obj = udisksclient.get_object(UDI)
block = obj.props.block

print("Creating partition table")
try:
    block.call_format_sync(
            'gpt',
            arg_options=GLib.Variant('a{sv}', None),
            cancellable=None)
except GLib.Error as e:
    # XXX: sometimes retrying fails as well
    # https://bugs.freedesktop.org/show_bug.cgi?id=76178
    if ('GDBus.Error:org.freedesktop.UDisks2.Error.Failed' in e.message and
            'Error synchronizing after initial wipe' in e.message):
        print("Failed to synchronize. Trying again, which usually solves the issue. Error was: %s" % e.message)
        subprocess.check_call('sync')
        time.sleep(5)
        block.call_format_sync(
                'gpt',
                arg_options=GLib.Variant('a{sv}', None),
                cancellable=None)

print("Creating partition")
obj = None
obj = udisksclient.get_object(UDI)
partition_table = obj.props.partition_table
try:
    partition_table.call_create_partition_sync(
            arg_offset=0,
            arg_size=2500 * 2**20,
            arg_type=ESP_GUID,
            arg_name="Test",
            arg_options=GLib.Variant('a{sv}', None),
            cancellable=None)
except GLib.Error as e:
    # XXX: as of Debian Jessie, we often get errors while wiping
    # (Debian bug #767457). We ignore them as they are not fatal
    # for us... but we need to fix a few things later...
    if ('GDBus.Error:org.freedesktop.UDisks2.Error.Failed' in e.message and
            'Error wiping newly created partition' in e.message):
        print("Ignoring error %s" % e.message)
    else:
        raise

# We still set the system partition flags using gdisk as udisks often
# fails to find the partition at this point
#self._set_partition_flags(self.drive['device'], 1, SYSTEM_PARTITION_FLAGS)

# subprocess.check_call('sync')
# time.sleep(5)

# Rescan the device as it seems this is not always done automatically.
# XXX: this should only be needed when call_create_partition_sync fails.
block.call_rescan_sync(GLib.Variant('a{sv}', None))

# Gets the new object after the rescan, then its 1st partition
# which is the one we just created on our new partition table.
obj = udisksclient.get_object(UDI)
partition_table = obj.props.partition_table
partitions = udisksclient.get_partitions(partition_table)
print("partitions: %(partitions)s" % {'partitions': partitions})
system_partition = partitions[0]

# Sets the partition type that is not set by call_create_partition_sync...
# because it is not implemented in UDisks as of version 2.1.5 (see
# http://cgit.freedesktop.org/udisks/tree/src/udiskslinuxpartitiontable.c?id=a66691c3#n630)
#
# XXX: this sometimes fails with:
# liveusb/creator.py, line 940, in partition_device
#    system_partition.call_set_type_sync(ESP_GUID,
#        GLib.Variant('a{sv}', None))
# Error: udisks-error-quark:
# GDBus.Error:org.freedesktop.UDisks2.Error.NotSupported:
# No support for modifying a partition a table of type `PMBR' (11)
system_partition.call_set_type_sync(ESP_GUID, GLib.Variant('a{sv}', None))
system_partition.call_set_name_sync("Test", GLib.Variant('a{sv}', None))
